@@ -88,6 +88,7 @@ sub spawn {
             _start         => \&_onpriv_start,
             _stop          => sub { warn "AI shutdown\n" },
             # public events
+            attack                   => \&_onpub_attack,
             place_armies     => \&_onpub_place_armies,
             place_armies_initial     => \&_onpub_place_armies_initial,
         },
@@ -121,6 +122,18 @@ sub description {
 
 # -- public events
 
+#
+# event: attack();
+#
+# request the ai to attack a country, or to end its attack turn.
+#
+sub _onpub_attack {
+    my $ai = $_[HEAP];
+    my ($action, @params) = $ai->attack;
+    K->post('risk', $action, @params);
+}
+
+
 #
 # event: place_armies($nb, $continent);
 #
@@ -232,6 +245,13 @@ An AI object will typically implements the following methods:
 
 =over 4
 
+=item * my ($action, [$from, $country]) = $ai->attack()
+
+Return the attack plan, which can be either C<attack> or C<attack_end> to stop
+this step of the ai's turn. If C<attack> is returned, then it should also
+supply C<$from> and C<$country> parameters to know the attack parameters.
+
+
 =item * my $str = $ai->description()
 
 Return a short description of the ai and how it works.
@@ -242,7 +262,7 @@ Return a short description of the ai and how it works.
 Return a difficulty level for the ai.
 
 
-=item * my @where = place_armies($nb, [$continent])
+=item * my @where = $ai->place_armies($nb, [$continent])
 
 Return a list of C<[ $country, $nb ]> tuples (a C<Games::Risk::Map::Country>
 and an integer) defining where to place C<$nb> armies. If C<$continent> (a
